PImage edgeDetection(PImage pic) {
  
  // Gauß-Maske für die Rauschreduzierung
  float[][] gauss = {{ 2,  4,  5,  4,  2},
                     { 4,  9, 12,  9,  4},
                     { 5, 12, 15, 12,  5},
                     { 4,  9, 12,  9,  4},
                     { 2,  4,  5,  4,  2} };
  float gaussFactor = 159;    // Vorfaktor der Gauss-Matrix

  // Sobel-Masken für die Kantenerkennung
  float[][] sobelX = {{ -1,  0,  1},
                      { -2,  0,  2},
                      { -1,  0,  1}};
                    
  float[][] sobelY = {{  1,  2,  1},
                      {  0,  0,  0},
                      { -1, -2, -1}};
  
  // Matrix für die Richtungen der Kanten
  int directions[] = new int[pic.width*pic.height];         
  
  filter(GRAY);
  image(pic, 0, 0);
  saveFrame("1-Schwarzweiss-Bild.png");
  
  PImage gaussPic = get();               // Bild nach gausscher Rauschentfernung
  PImage sobelPic = get();               // Addition von sobelX und sobelY
  PImage thinEdgesPic = get();           // Bild mit dünnen Kanten
  PImage finalPic = get();               // Bild mit schwarzem Rand
  PImage hystPic = get();
  
  gaussPic.loadPixels();
  sobelPic.loadPixels();
  thinEdgesPic.loadPixels();
  finalPic.loadPixels();
  hystPic.loadPixels();
  clear();
  
  
  // GAUSSFILTER ANWENDEN
  // Die Gauss-Matrix wird über den gewählten Pixel gelegt und mithilfe der Einträge ein gewichteter Mittelwert gebildet, 
  // der der neue Pixelwert wird: Dazu wird jeder Pixel in der 5x5-Umgebung mit dem jeweilligen Matrixeintrag/115 multipliziert 
  // und am Ende alle summiert.
  
  for (int y=2; y<pic.height-3; y++) {                       // vernachlässigt den Bildrand, damit die Gaussmatrix ganz raufpasst
      for (int x=2; x<pic.width-3; x++) {
          float gaussValue = 0;                              // gewichtetes Mittel für den Pixel (x,y)
          for (int yGauss = -2; yGauss <= 2; yGauss++) {     // Schleifen für den 5x5-Bereich, der von der Gaussmatrix überdeckt wird
              for (int xGauss = -2; xGauss <=2; xGauss++) {  
                  int position = (y+yGauss)*pic.width + x+xGauss;    //Position des zu bearbeitenden Pixels
                  gaussValue += gauss[xGauss+2][yGauss+2] * red(pic.pixels[position]);    // da das Bild schwarz-weiß ist, sind red/green/blue identisch
              }
          }
          gaussValue = int(gaussValue / gaussFactor);
          gaussPic.pixels[y*pic.width + x]= color(gaussValue, gaussValue, gaussValue);    //wenn r,g und b den gleichen Wert haben, ist die Farbe grau
      }
  }
  pic = gaussPic;  
  
  image(pic, 0,0);
  saveFrame("2-GaussBild.png");
  clear();
  
  
  // SOBELMASKEN ANWENDEN, RICHTUNGEN BESTIMMEN
  // Die Matrizen werden über den gewählten Pixel gelegt und ein gewichteter Mittelwert wird für beide Matrizen gebildet. 
  // Durch SobelX wird der Gradient jedes Pixels in x-Richtung, durch SobelY in y-Richtung errechnet. 
  // Am Ende werden die Beträge addiert.
  
  for (int y=1; y<pic.height-2; y++) {                      // vernachlässigt den Bildrand, damit die Sobel-Matrizen ganz raufpassen
      for (int x=1; x<pic.width-2; x++) {
          float sobelXValue = 0;                            // Gradient in x-Richtung für den Pixel (x,y)
          float sobelYValue = 0;                            // Gradient in y-Richtung für den Pixel (x,y)
          for (int ySobel = -1; ySobel <= 1; ySobel++) {    // For-Schleifen für den 3x3-Bereich, der von Sobelmatrizen überdeckt wird
              for (int xSobel = -1; xSobel <=1; xSobel++) {  
                  int position = (y+ySobel)*pic.width + x+xSobel;
                  sobelXValue += sobelX[xSobel+1][ySobel+1] * red(pic.pixels[position]);
                  sobelYValue += sobelY[xSobel+1][ySobel+1] * red(pic.pixels[position]);
              }
          }
          int sobelValue = int(abs(sobelXValue) + abs(sobelYValue));
          
          // 255 ist der größtmögliche Farbwert, deshalb dürfen die Sobel-Werte ihn nicht überschreiten:
          if (sobelValue > 255) {
            sobelValue = 255;
          }          
          if (sobelXValue > 255) {
            sobelXValue = 255;
          }
          if (sobelYValue > 255) {
            sobelYValue = 255;
          }
          sobelPic.pixels[y*pic.width + x]= color(sobelValue, sobelValue, sobelValue);
          
          // Winkel berechnen (wird später bei der Verdünnung der Kanten benötigt):
          // sobelXValue und sobelYValue bilden ein rechtwinkliges Dreieck. Der Winkel des Gradienten wird über den Tangens bestimmt.

          float angle;
          if (sobelXValue == 0 && sobelYValue == 0) {              // waagrechte Kante = 0
              angle = 0;
          } else if (sobelXValue == 0 && sobelYValue != 0) {       // senkrechte Kante = 1
              angle = 1;
          } else {
              angle = atan(sobelXValue/sobelYValue)*360/(2*PI);
              if (angle > -22.5 && angle < 22.5) {
                  angle = 0;
              } else if (angle >= 22.5 && angle < 67) {
                  angle = 2;                                     // schräg nach oben (45°) = 2
              } else if (angle <= -22.5 && angle > -67) {
                  angle = 3;                                     // schräg nach unten (-45°) = 3
              } else {
                  angle = 1;
              }
          }
          directions[y*pic.width+x] = int(angle);        
      }
  }
  pic = sobelPic;

  image(pic, 0, 0);
  saveFrame("3-SobelBild.png");
  clear();


  // KANTEN DÜNNER MACHEN
  // Die Kanten werden über die Non-Maximum-Suppression einen Pixel breit gemacht
  // dazu werden für jeden Pixel seine Nachbarpixel in Gradientenrichtung betrachtet. Ist einer der beiden Nachbarn heller,
  // wird der betrachtete Pixel auf 0 gesetzt.

  for (int y=1; y<pic.height-2; y++) {                     
      for (int x=1; x<pic.width-2; x++) {
          int current = y*pic.width+x;    // aktueller Pixel
          // Liste der Nachbar-Paare in den 4 Richtungen 0°, 90°, 45° und -45°
          int neighbors[][] = {{y*pic.width+x+1,     y*pic.width+x-1},        // Index 0 = 0° Nachbarn
                               {(y-1)*pic.width+x,   (y+1)*pic.width+x},      // Index 1 = 90° Nachbarn
                               {(y-1)*pic.width+x+1, (y+1)*pic.width+x+1},    // Index 2 = 45° Nachbarn
                               {(y+1)*pic.width+x+1, (y-1)*pic.width+x-1}};   // Index 3 = -45° Nachbarn
          int angle = directions[current];    // aktueller Gradientenwinkel

          if ((red(pic.pixels[current]) < red(pic.pixels[neighbors[angle][0]]))     // Pixel dunkler als Nachbar 0 in Gradientenrichtung?
          || (red(pic.pixels[current]) < red(pic.pixels[neighbors[angle][1]]))) {   // Pixel dunkler als Nachbar 1 in Gradientenrichtung?
              thinEdgesPic.pixels[current] = color(0,0,0);
          } else {
              thinEdgesPic.pixels[current] = pic.pixels[current];
          }
      }
  }
  pic = thinEdgesPic;
  
  image(pic, 0, 0);
  saveFrame("4-duennesBild.png");
  clear();
  
  // RANDPIXEL SCHWARZ MACHEN:
  for (int y=0; y<pic.height; y++) {
      for (int x=0; x<pic.width; x++) {
          int current = y*pic.width+x;
          if (x<5 || y<5) {
              finalPic.pixels[current]=color(0,0,0);
          } else if ((pic.width-x)<5 ||(pic.height-y)<5)  {
              finalPic.pixels[current]=color(0,0,0);
          } else {
              finalPic.pixels[current]=pic.pixels[current];
          }
      }
  }
  pic = finalPic;
  

  // RÜCKGABE
  image(pic, 0, 0);
  filter(THRESHOLD, 0.2);  // binärer Schwarz-Weiß-Filter
  saveFrame("5-finalesBild.png");
  PImage edges = get();
  clear();
  return edges;
}